[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Conversational API Powered by FactSet Mercury client library for Python

[![API Version](https://img.shields.io/badge/api-v1.1.1-blue)](https://developer.factset.com/api-catalog/conversational-api-powered-factset-mercury)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.ConversationalAPIPoweredbyFactSetMercury/1.2.0)](https://pypi.org/project/fds.sdk.ConversationalAPIPoweredbyFactSetMercury/v/1.2.0)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

### OVERVIEW

The FactSet Conversational API allows clients to integrate core FactSet Mercury capabilities in chatbot experiences.

The Conversational API is powered by FactSet Mercury, FactSet's Generative Artificial Intelligence (GenAI) large language model. The Conversational API provides a variety of content and capabilities, including FactSet’s Federation layer (FactSet’s core GenAI-based technology), as well as more specific content and functionality tailored for financial services workflows.

The Conversational API provides answers to hundreds of natural language search queries and allows you to easily ask questions related to companies and markets research.

Some example supported prompts:

- Nintendo's highest closing stock price over the last 3 months
- Has Yelp issued any guidance?
- What are the key trends impacting costs for DaVita?

**For Information on Access to and Content Available via the Conversational API**

Please see the [Conversational API Online Assistant Page](https://my.apps.factset.com/oa/pages/23209). Here you can find instructions on how to set up access to the Conversational API, and the full list of content available.

#### Conversational API Consumer Workflow

The Conversational API is an asynchronous API that utilizes status polling to inform the consumer when a query response is complete. Please see the technical OpenAPI documentation below for specific information regarding consuming the API programmatically.

At a high level, the API consumer workflow is as follows:

1) Send a natural language query to the `/query` endpoint and start the response generation process.
2) Poll the status of the response generation process using the `/status` endpoint.
3) Once the status indicates a ready response, retrieve it using the `/result` endpoint. 
- If your response contains a file ID, such as for an Excel chart or a [FactSet ActiveGraph](https://my.apps.factset.com/oa/pages/20355), retrieve it using the file ID at the `/download/file` endpoint.
- To provide feedback on your response and help the Conversational API better serve you content, we encourage you to use the `/feedback` endpoint. 

#### Receive updates on changes to the Conversational API
Subscribe to our FactSet Notify by clicking \"Subscribe to notifications\" in the upper right above the API overview. You will receive email updates when any updates occur to the API

#### FAQ & Troubleshooting
Please find FAQs and troubleshooting tips in the [FAQs section](https://my.apps.factset.com/oa/pages/23440) of the Conversational API OA page

##### Current Limitations

- \"Natural language\" in this documentation refers to modern conversational English. Support for other languages is currently unavailable.
- The Conversational API is currently limited to accept 10 natural language queries per minute and 200 per hour for an individual consumer. If you anticipate your needs to be greater than these limits, please reach out to FactSet Support.


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.1.1
- SDK version: 1.2.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.ConversationalAPIPoweredbyFactSetMercury==1.2.0
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.ConversationalAPIPoweredbyFactSetMercury==1.2.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient
import fds.sdk.ConversationalAPIPoweredbyFactSetMercury
from fds.sdk.ConversationalAPIPoweredbyFactSetMercury.api import chat_api
from fds.sdk.ConversationalAPIPoweredbyFactSetMercury.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.ConversationalAPIPoweredbyFactSetMercury.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.ConversationalAPIPoweredbyFactSetMercury.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.ConversationalAPIPoweredbyFactSetMercury.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = chat_api.ChatApi(api_client)
    query_chat_request = QueryChatRequest(
        data=QueryChatRequestData(
            query="query_example",
            chat_id="chat_id_example",
        ),
    ) # QueryChatRequest | Request body to the `/query` endpoint, containing the natural language query to be sent to FactSet Mercury and an optional chat ID for continuing an ongoing conversation.

    try:
        # Send a natural language query to FactSet Mercury
        # example passing only required values which don't have defaults set
        api_response = api_instance.send_query(query_chat_request)

        pprint(api_response)
    except fds.sdk.ConversationalAPIPoweredbyFactSetMercury.ApiException as e:
        print("Exception when calling ChatApi->send_query: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Send a natural language query to FactSet Mercury
    #     api_response, http_status_code, response_headers = api_instance.send_query_with_http_info(query_chat_request)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.ConversationalAPIPoweredbyFactSetMercury.ApiException as e:
    #     print("Exception when calling ChatApi->send_query: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Send a natural language query to FactSet Mercury
    #     async_result = api_instance.send_query_async(query_chat_request)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.ConversationalAPIPoweredbyFactSetMercury.ApiException as e:
    #     print("Exception when calling ChatApi->send_query: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Send a natural language query to FactSet Mercury
    #     async_result = api_instance.send_query_with_http_info_async(query_chat_request)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.ConversationalAPIPoweredbyFactSetMercury.ApiException as e:
    #     print("Exception when calling ChatApi->send_query: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.ConversationalAPIPoweredbyFactSetMercury

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.ConversationalAPIPoweredbyFactSetMercury.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.ConversationalAPIPoweredbyFactSetMercury

configuration = fds.sdk.ConversationalAPIPoweredbyFactSetMercury.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.ConversationalAPIPoweredbyFactSetMercury

configuration = fds.sdk.ConversationalAPIPoweredbyFactSetMercury.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.ConversationalAPIPoweredbyFactSetMercury

configuration = fds.sdk.ConversationalAPIPoweredbyFactSetMercury.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/conversational/v1*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*ChatApi* | [**get_chat_result**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ChatApi.md#get_chat_result) | **POST** /result | Retrieve the completed response for your query
*ChatApi* | [**get_chat_status**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ChatApi.md#get_chat_status) | **POST** /status | Retrieve the status of a chat response generation process
*ChatApi* | [**send_query**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ChatApi.md#send_query) | **POST** /query | Send a natural language query to FactSet Mercury
*DownloadApi* | [**download_file**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/DownloadApi.md#download_file) | **POST** /download/file | Retrieve data file based on file ID.
*FeedbackApi* | [**send_feedback**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/FeedbackApi.md#send_feedback) | **POST** /feedback | Send feedback on the response to a query.


## Documentation For Models

 - [AdaptiveCard](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/AdaptiveCard.md)
 - [BadFeedbackRequestError](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/BadFeedbackRequestError.md)
 - [BadFeedbackRequestErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/BadFeedbackRequestErrorObject.md)
 - [BadRequestError](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/BadRequestError.md)
 - [BadRequestErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/BadRequestErrorObject.md)
 - [BadRequestErrorWithInvalidDataSchema](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/BadRequestErrorWithInvalidDataSchema.md)
 - [BadRequestErrorWithInvalidDataSchemaObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/BadRequestErrorWithInvalidDataSchemaObject.md)
 - [ChatPollingRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ChatPollingRequest.md)
 - [ChatPollingRequestData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ChatPollingRequestData.md)
 - [Citations](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/Citations.md)
 - [Download](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/Download.md)
 - [FederationData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/FederationData.md)
 - [FeedbackRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/FeedbackRequest.md)
 - [FeedbackRequestData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/FeedbackRequestData.md)
 - [FileDownloadRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/FileDownloadRequest.md)
 - [FileDownloadRequestData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/FileDownloadRequestData.md)
 - [FileGenerationErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/FileGenerationErrorObject.md)
 - [ForbiddenError](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ForbiddenError.md)
 - [ForbiddenErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ForbiddenErrorObject.md)
 - [InternalServerError](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/InternalServerError.md)
 - [InternalServerErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/InternalServerErrorObject.md)
 - [MarkdownTable](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/MarkdownTable.md)
 - [MarkdownTableMetadata](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/MarkdownTableMetadata.md)
 - [NextStep](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/NextStep.md)
 - [NextStepItem](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/NextStepItem.md)
 - [OpenUrl](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/OpenUrl.md)
 - [OpenUrlUrl](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/OpenUrlUrl.md)
 - [Phrase](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/Phrase.md)
 - [PromptItem](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/PromptItem.md)
 - [PromptParameter](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/PromptParameter.md)
 - [PromptParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/PromptParameters.md)
 - [QueryChatRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/QueryChatRequest.md)
 - [QueryChatRequestData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/QueryChatRequestData.md)
 - [QueryChatResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/QueryChatResponse.md)
 - [QueryChatResponseData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/QueryChatResponseData.md)
 - [QueryResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/QueryResponse.md)
 - [QueryResponseData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/QueryResponseData.md)
 - [ResourceNotFoundError](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ResourceNotFoundError.md)
 - [ResourceNotFoundErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ResourceNotFoundErrorObject.md)
 - [ResponseText](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/ResponseText.md)
 - [SpeakerInfo](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/SpeakerInfo.md)
 - [StatusPollResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/StatusPollResponse.md)
 - [StatusPollResponseData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/StatusPollResponseData.md)
 - [SuggestedPrompts](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/SuggestedPrompts.md)
 - [Table](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ConversationalAPIPoweredbyFactSetMercury/v1/docs/Table.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.ConversationalAPIPoweredbyFactSetMercury.apis and fds.sdk.ConversationalAPIPoweredbyFactSetMercury.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.ConversationalAPIPoweredbyFactSetMercury.api.default_api import DefaultApi`
- `from fds.sdk.ConversationalAPIPoweredbyFactSetMercury.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.ConversationalAPIPoweredbyFactSetMercury
from fds.sdk.ConversationalAPIPoweredbyFactSetMercury.apis import *
from fds.sdk.ConversationalAPIPoweredbyFactSetMercury.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

